#!/usr/bin/perl -w

##
##  This file is part of The NWA Toolset.
##
##  Copyright (C) 2001-2002 Royal Library in Stockholm,
##                          Royal Library in Copenhagen,
##                          Helsinki University Library of Finland, 
##                          National Library of Norway,
##                          National and University Library of Iceland.
##
##  The NWA Toolset is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  The NWA Toolset is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with The NWA Toolset; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##

# $Id: generate_identifiers.pl,v 1.2 2004/10/18 21:38:46 sverreb Exp $
# Function: to create file of archive identifiers for input to exporter.pl
## in the format specified by XML Schema aidlist.xsd

use Getopt::Std;
use File::Find;
use Cwd;
use strict;

my $output_prefix = undef;
my $outputfile_number = 0;
my $output_filename=undef;
my $directory=undef;
my $dirhandle=undef;
my %options = ();
my $aid = undef;
my $document_count = 0;
my $document_in_file_count = 0;
my $recursive = 0;
my $max_ids_per_file = 100000;  # the maximum number of identifiers in each outputfile.

# parse the command line for the options 
#   -h                           # Show script-usage and exit 
#   -d <input-directory>,        # Specify inputdirectory. Default is current working directory
#   -o <output-prefix>           # Specify output-prefix. First outputfile has suffix -0, then -1 and so on.
#   -r                           # Generate archive-identifiers recursively"; 
#   -n <number_of_ids_per_file>  # Specify the maximum number of identifiers in each outputfile. Default is 100.000\n";
#   
  
# Note ':' after an option-letter, tells the program, that this option (i.e. '-d' and '-o') comes with an argument
getopts("hd:o:rn:", \%options); 

if (defined($options{h})){ # print out usage

    &print_usage();
    exit();
}

if (!defined($options{d})){ # Try to read option -d
    $directory = cwd();
    print "\ngenerate_identifiers.pl[WARNING]: OPTION -d <directory> not set. Using CWD ($directory) as input directory";
    
}
else {
    $directory = $options{d};
}

if (!defined($options{o})){
    print "\ngenerate_identifiers.pl[FATAL]: OPTION -o <output-prefix> not set. Program aborted!";

    &print_usage();
    exit();
}
else {
    $output_prefix = $options{o};
}

if (defined($options{r})) {
    $recursive = 1;
}

if ($directory =~ /\/$/) {
    $directory =~ s/\/$//; 
}

if (defined($options{n})) {
    $max_ids_per_file = $options{n};
    print "\nNB: max_ids_per_file set to $max_ids_per_file";
    if ($max_ids_per_file <= 0) { # Check,if option -n is greater than zero
	$max_ids_per_file = 100000;
	print "\ngenerate_identifiers.pl[WARNING]: OPTION -n was less than 0. Set to the default: 100000";
    }
}
else {
    print "\nThe maximum number of identifiers in each outputfile set to 100.000\n";
}


$output_filename = $output_prefix .  "-$outputfile_number";

open(OUTPUTFILE,">$output_filename") ||
                die "Unable to open $output_filename for writing :$!";

print OUTPUTFILE "<?xml version='1.0' encoding='ISO-8859-1'?>\n";
print OUTPUTFILE "<aidlist>\n";

process_dir($directory);

print OUTPUTFILE "</aidlist>\n";

close(OUTPUTFILE);

print "\n\nFinished writing $document_in_file_count IDs to AID-file: $output_filename\n";
print "\nFound $document_count documents in total. \n";
print "\nDone!\n";


sub process_dir {
    my ($curdir) = @_;
    my $aid;
    opendir(AID_DIR, $curdir) or die "Can't open dir: $directory: $!";
    my @aids = readdir(AID_DIR);
    closedir(AID_DIR);

    print "\nWorking with: $curdir";
    foreach $aid (@aids) {
	next if $aid =~ /^\.\.?$/;   # skip . and ..
	next if $aid =~ /\.meta$/;   # skip .meta files
	if (-d "$curdir/$aid") {
	    process_dir("$curdir/$aid") if $recursive;
	    next;
	}
	#next unless -f "$curdir/$aid";
	#print "FILE: $aid\n";
		
	$document_in_file_count++; # wait with the printing to file, before we know, if this is the time
	# to start with a new outputfile
        $document_count++;
	if ($document_in_file_count > $max_ids_per_file) {
	    $document_in_file_count--;
	    print "\n\nFinished writing $document_in_file_count IDs to AID-file: $output_filename\n";
    
	    # finish and close current outputfile
	    print OUTPUTFILE "</aidlist>\n";
	    close(OUTPUTFILE);
	    # open and start new outputfile
	    $outputfile_number++;
	    $output_filename = $output_prefix .  "-$outputfile_number";
	    open(OUTPUTFILE,">$output_filename") ||
                die "Unable to open $output_filename for writing :$!";
	    print OUTPUTFILE "<?xml version='1.0' encoding='ISO-8859-1'?>\n";
	    print OUTPUTFILE "<aidlist>\n";

	    $document_in_file_count=1; # reset file-count to one.
	}
	print OUTPUTFILE "<aid>$curdir/$aid</aid>\n";    
    }
}

sub print_usage()
{
    print "\n--------------------";
    print "\nProgram: generate_identifiers.pl\n";
    print "--------------------";
    printf "\nFunction: to create file of archive identifiers for input to exporter.pl in the format \nspecified by the XML Schema aidlist.xsd.\n\n";

    print "Usage: perl generate_identifiers.pl -o <output-prefix> [-r] [-d <inputdirectory>] [-n <number_of_ids_per_file>]";
    print "\nUsage: perl generate_identifiers.pl -h";
    print "\n\t -r: generate archive-identifiers recursively"; 
    print "\n\t -h: Print out this message";
    print "\n\t -o <output-prefix>: Specify outputfilename-prefix. First outputfile has suffix -0, then -1 and so on.";
    print "\n\t -d <directory>: Specify inputdirectory. Default is current working directory\n";
    print "\n\t -n <number_of_ids_per_file>: Specify the maximum number of identifiers in each outputfile. Default is 100.000\n";
    
}
1;
