<?php
/*
 *  This file is part of The NWA Toolset.
 *
 *  Copyright (C) 2001-2002 Royal Library in Stockholm,
 *                          Royal Library in Copenhagen,
 *                          Helsinki University Library of Finland, 
 *                          National Library of Norway,
 *                          National and University Library of Iceland.
 *
 *  The NWA Toolset is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The NWA Toolset is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with The NWA Toolset; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
* The IndexSearch class defines methods
* for interfacing "any" search engine
*
* To interface your own search engine
* use this class as the superclass.
*
* $Id: indexSearch.inc,v 1.1.1.1 2004/02/04 14:29:09 sverreb Exp $
*/

class indexSearch {
  var $query;
  var $numhits;
  var $numhitstotal;
  var $hitsperset;
  var $offset;
  var $timespent;
  var $resultfields = array();
  var $resultset = array();
  var $sortorder;
  var $errormsg;
 
  /**
  * Set the result set size
  *
  * Sets the number of hits returned by the 
  * getResultSet and getXmlResultSet methods
  *
  * @param  integer  Number of hits to return
  */  
  function setSizeOfResultSet($hits) {
    $this->hitsperset = $hits;
  }
  
  /**
  * Set the offset 
  *
  * Sets the offset for the result set. An offset of 10
  * and a result set size of 10 will give a result 
  * set with hits 11-20 (assuming the hitnumbers 
  * start with 1)
  *
  * @param  integer  Offset
  */  
  function setOffset($offset) {
    $this->offset=$offset;
  }
  
  /** 
  * Set the sort order 
  *
  * @param  string  Sort order
  */
  function setSortOrder($sortorder) {
    $this->sortorder=$sortorder;
  } 

  /**
  * Get the query response time
  * 
  * @return  float  Time spent on query
  */
  function getTimeSpent() {
    return $this->timespent;
  } 

  /** 
  * Get number of hits in resultset
  *
  * @return number of hits
  */
  function getNumHits() {
    return $this->numhits;
  }

  /** 
  * Get the total number of hits
  *
  * @return number of hits
  */
  function getNumHitsTotal() {
    return $this->numhitstotal;
  }
  
  
  /**
  * Set query string for search
  *
  * The query string must conform to the syntax
  * outlined below. The query string may be in 
  * upper, lower or mixed case.
  * 
  * Searching text indices:
  *  schema.index:"word" - Word match
  *  schema.index:"word*" - Right hand truncation, match for part of word
  *  schema.index:"phrase" - Phrase match
  * 
  * Searching integer indices:
  *  schema.index:i - Exact match
  *  schema.index:[i;j] - Range(including boundaries)
  *  schema.index:[i;] - Larger than or equal to
  *  schema.index:[;i] - Smaller than or equal to
  *  schema.index:>i - Larger than
  *  schema.index:<i - Smaller than
  * 
  * In the context of the NWA Access Module schema will be NWA. 
  * The index names will be defined by the NWA document format 
  * (http://nwa.nb.no/nwa/export/1.0/).
  * 
  * Boolean operators:
  * The NWA query language supports the boolean operators: AND, OR, ANDNOT
  * 
  * Parenthesis:
  * The left parenthesis "(" and right parenthesis ")" are used 
  * to dilineate one expression from another. For example, in the query 
  *
  *   nwa.dcdate:998690406 AND (nwa.title:"ouagadougou" 
  *   OR nwa.title:"capitol of burkina fazo"), 
  *
  * parentheses group the ORs together so they are as a distinct 
  * entity from the AND.
  * 
  * @param  string  Query string
  */
  function setQuery($querystring) {
    $this->query = $querystring;
  }

  /**
  * Sets which fields to include in the result
  * set returned by the getResultSet and 
  * getXmlResultSet methods.
  *
  * $fields : a comma, or whitespace limited 
  * list of result fields.
  *
  */
  function setFieldsInResult($fields) {
  }
  
  /**
  * Executes the query.
  *
  * This method will populate the result set array
  * Returns an array of the following form:
  *   Array([0]=>Array([dcdate]=>"2001-10-14"[dctitle]=>"Some title")
  *         [1]=>Array([dcdate]=>"2001-10-15"[dctitle]=>"Another title"))
  * the fields returned in the array is determined by the 
  * setResultFields method (e.g. setResultFields("dcdate, dctitle")
  *
  * @return boolean False if error
  */
  function doQuery() {
    return true;
  }
  
  /**
  * Get the result set
  *
  * @return  array  Result set
  */
  function getResultSet() {
    return $this->resultset;
  }  

  /**
  * If the doQuery method returned false
  * use this to fetch the error message string
  *
  * @return  string Error message
  */
  function getErrorMessage() {
    return $this->errormsg;
  }  

  /**
  * Get the result set
  *
  * The method will return an xml formatted result of the following form:
  * 
  * <?xml version="1.0" encoding="utf-8"?>
  * <resultset query="query" sortorder="descending" fields="dcdate, dctitle" hits=10 totalhits="112" spanstart="1" spanend="10" timespent="0.0017489194869995">
  *   <doc id="1">
  *     <dcdate></dcdate>
  *     <dctitle></dctitle>
  *   </doc>
  *   <doc id="2">
  *     <dcdate></dcdate>
  *     <dctitle></dctitle>
  *   </doc>  
  *  ..
  *  .
  * </resultset>
  *
  * @return  string  Result set
  */
  function getXmlResultSet() {
    global $HTTP_SERVER_VARS;
    $address = "http://" . $HTTP_SERVER_VARS["HTTP_HOST"] . $HTTP_SERVER_VARS["SCRIPT_NAME"];
    $resultsetarray=$this->getResultSet();
    $spanstart=$this->offset+1;
    if ($this->numhitstotal==0) {
      $spanend="";
      $spanstart="";
    }
    elseif ($this->numhits < $this->hitsperset) {
      $spanend=$this->offset+$this->numhits;
    }
    else {
      $spanend=$this->offset+$this->hitsperset;    
    }
    $retval='<?xml version="1.0" encoding="utf-8"?>'."\n";
    $retval.=sprintf('<resultset address="%s" query="%s" sortorder="%s" fields="%s" hits="%d" totalhits="%d" hitsperset="%d" offset="%d" spanstart="%d" spanend="%d" timespent="%s">', $address, $this->query, $this->sortorder, implode(" ", $this->resultfields), $this->numhits, $this->numhitstotal, $this->hitsperset, $this->offset, $spanstart, $spanend, $this->timespent) . "\n";    
    
    $i=$this->offset;
    while (list($setkey,$result)=each($this->resultset)) {
      $retval.='  <doc id="'.++$i.'">'."\n";
      while (list($key,$val)=each($result)) {
        $retval.="    <".$key.">".$val."</".$key.">\n";
      }
      $retval.="  </doc>\n";
    }        
    $retval.="</resultset>\n";
    return $retval; 
  }
}
?>
